import snappy
import math
from math import sqrt, pi
import cmath
from cmath import log, exp
from datetime import datetime
import copy

# constants.py contains the basic constants used here

from constants import *

# functions.py contains the basic numerical functions used here

from functions import *

# manifold.py contains functions that use the manifold class

from manifold import *

# A 'slope' is a pair (p,q) of coprime integers with q>=0, 
# with (-1,0) excluded (because (1,0) suffices). Slopes are in 1-1 correspondence
# with esssential simple curves on a torus, after fixing a homology basis (as usual).
# A 'filling' is a sequence of slopes or (0, 0) that (partially) fill a manifold. 
# The element (0, 0) denotes an unfilled cusp, as in Snappy.
# For instance, [(-1,1), (0,0), (7,3)] is a filling on a 3-cusped manifold.

# This is the main list of fillings. The program looks for a geometric solution
# of each element in the list. 

fillings = []

# The program tries to find a geometric solution on all covers having degree<=max_degree.
# Computer time grows more than exponentially with max_degree ! 7 is fine, 11 is very slow

max_degree = 9

# Here you must set the manifold N you want to investigate. 
# Some examples are listed below.

# N = snappy.Manifold('4_1')             	# The figure-8 knot complement
# N = snappy.Manifold('Whitehead.tri')            #The Whitehead link complement
# N = snappy.Manifold('chain3.tri')      #The magic manifold (chain link with 3 components)
# N = snappy.Manifold('chain4.tri')      #The chain link with 4 components with vol = 2octahedra
# N = snappy.Manifold('chain5_fill_m2.tri')      #The chain link with 5 components filled with -2
# N = snappy.Manifold('chain6_fill_2.tri')      #The chain link with 6 components filled with 2
# N = snappy.Manifold('chain7_fill_1.tri') # The chain link with 7 components filled with 1
# N = snappy.Manifold('chain7_fill_m2.tri') # The chain link with 7 components filled with -2
# N = snappy.Manifold('chain7_fill_2.tri') # The chain link with 7 components filled with 2
# N = snappy.Manifold('chain7_fill_m3.tri') # The chain link with 7 components filled with -3
# N = snappy.Manifold('chain7_fill_m12.tri') # The chain link with 7 components filled with -1/2
# N = snappy.Manifold('chain7_fill_12.tri') # The chain link with 7 components filled with 1/2
# N = snappy.Manifold('chain6.tri')             	
N = snappy.Manifold('chain7.tri')             	

# The main program starts here

time1 = datetime.now()

if verbose:
	print 'Manifold volume:'
	print N.volume()
	print 'Cusp shapes:'
	print N.cusp_info()

num_cusps = N.num_cusps()

# The fillings for which we could detect a positive solution are stored here

hyperbolic_fillings = []

# The others are stored here

no_answer = []

print "Number of fillings to analyse:", len(fillings)
counter = 1

for filling in fillings:
	print counter, filling
	counter = counter + 1
	M = N.copy()
	M.dehn_fill(filling)
	flag, M2, tetrahedra_shapes = try_to_hyperbolize(M, number_of_randomizes, try_permanent_fillings, min_imaginary)
	if flag == False:
		for degree in range(max_degree+1):
			if degree>1:
				covers = M.covers(degree)
				if verbose >=1:
					print 'Number of covers with degree ', degree, ':', len(covers)
				for Mcover in covers:
					flag, M2, tetrahedra_shapes = try_to_hyperbolize(Mcover, number_of_randomizes, try_permanent_fillings, min_imaginary)
					if flag:
						break
			if flag:
				break
	if flag: 
		hyperbolic_fillings.append(filling[:])
		if verbose:
			print 'has a positive solution'
	else:
		no_answer.append(filling[:])
		if verbose:
			print 'I cannot find a positive solution'

# Print the results
print 'The following fillings have a (possibly virtual) geometric solution:'
for filling in hyperbolic_fillings:   
	print filling
print 'Total:', len(hyperbolic_fillings)
	
if len(no_answer) > 0:
	print 'I cannot find a geometric solution for the following fillings:'
	for filling in no_answer:   
		print filling
	print 'Total:', len(no_answer)
		
time2 = datetime.now()
if verbose:
	print 'Computer time needed:', time2 - time1